SellOMatic_List = LibStub("AceAddon-3.0"):NewAddon("SellOMatic_List", "AceEvent-3.0", "AceConsole-3.0");
local SellOMatic = _G.SellOMatic
local item_tooltip = LibStub("LibGratuity-3.0");

local function getOption(info)
	return (info.arg and SellOMatic.db.profile[info.arg] or SellOMatic.db.profile[info[#info]]);
end;

local function setOption(info, value)
	local key = info.arg or info[#info];
	SellOMatic.db.profile[key] = value;
end;

local list_options = {
	type = "group",
	name = "List Options",
	args = {
		caseSensitiveList = {
				name = "Case sensitive list",
				desc = "Toggle ON/OFF case sensitive lists.",
				order = 2,
				type = "toggle",
				get = getOption,
				set = setOption,
		},
		list_allowBoP = {
			name = "Sell BoP",
			desc = "Toggle ON/OFF selling green soulbound items.",
			order = 3,
			type = "toggle",
			get = getOption,
			set = setOption,
		},
		list_allowBoE = {
			name = "Sell BoE",
			desc = "Toggle ON/OFF selling green bind on equip items.",
			order = 4,
			type = "toggle",
			get = getOption,
			set = setOption,
		},
		list_allowWhite = {
			name = "Sell common",
			desc = "Toggle ON/OFF selling common (white) items.",
			order = 5,
			type = "toggle",
			get = getOption,
			set = setOption,
		},
		list_allowGreen = {
			name = "Allow Green",
			desc = "Toggle ON/OFF selling common (white) items.",
			order = 6,
			type = "toggle",
			get = getOption,
			set = setOption,
		},
		list_allowBlue = {
			name = "Allow Blue",
			desc = "Toggle ON/OFF selling blue quality items.",
			order = 7,
			type = "toggle",
			get = getOption,
			set = setOption,
		},
		list_allowEpic = {
			name = "Allow Epic",
			desc = "Toggle ON/OFF selling epic quality items.",
			order = 8,
			type = "toggle",
			get = getOption,
			set = setOption,
		},
		list_useILevel = {
			name = "Use iLevel",
			desc = "Toggles ON/OFF selling items by iLevel.",
			order = 9,
			type = "toggle",
			get = getOption,
			set = setOption,
		},
		list_iLevelValue = {
			name = "iLevel value",
			desc = "Items below the iLevel value set will be sold",
			order = 10,
			type = "range",
			min = 1,
			max = 200,
			step = 1,
			get = getOption,
			set = setOption,
		},
	},
}

function SellOMatic_List:OnInitialize()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SellOMatic_List", list_options);
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SellOMatic_List", "List Options", "Sell-O-Matic");
end;

function SellOMatic:Scan_List()
	SellOMatic_List:Check_SellList();
	SellOMatic_List:Check_SaveList();
	--if SellOMatic.db.profile.list_useILevel then SellOMatic_List:Apply_iLevel() end;
end;

function SellOMatic_List:Check_SellList()
	local bag, slot, name, itemName, itemLink, itemRarity, itemSellPrice, itemStackCount, add;
	for bag = 0,4,1 do
		for slot = 1, GetContainerNumSlots(bag), 1 do
			add = 0;
			name = GetContainerItemLink(bag,slot);
			if name ~= nil then
				itemName, itemLink, itemRarity, itemLevel, _, _, _, itemStackCount, _, _, itemSellPrice = GetItemInfo(name);
				item_tooltip:SetBagItem(bag,slot);
				if itemRarity == 1 then
					if SellOMatic_List:Check_Item(SellOMatic.db.profile.sellList,itemName) == 1 then
						add = 1;
					end;
				elseif itemRarity > 1 and item_tooltip:Find(ITEM_BIND_ON_EQUIP,1,3) and SellOMatic.db.profile.allowBoE or item_tooltip:Find(ITEM_SOULBOUND,1,3) and SellOMatic.db.profile.allowBoP then
					if SellOMatic_List:Check_Item(SellOMatic.db.profile.sellList,itemName) == 1 then
						add = 1;
					end;
				end;
				if add == 1 and SellOMatic.db.profile.list_iLevelValue then
					if itemLevel < SellOMatic.db.profile.list_iLevelValue then add = 0 end;
				end;
				if add == 1 then
					if itemStackCount > 1 then
						itemSellPrice = itemSellPrice * GetItemCount(itemName);
					end;
					SellOMatic:Add_Shopping_List(itemName, itemLink, itemRarity, itemLevel, itemSellPrice);
				end;
			end;
		end;
	end;
end;

function SellOMatic_List:Check_SaveList()
	if SellOMatic.db.profile.saveList[1] ~= nil then
		local list = SellOMatic:Get_Shopping_List();
		local i;
		for i=1, #list, 1 do
			if SellOMatic_List:Check_Item(SellOMatic.db.profile.saveList,list[i][1]) then
				SellOMatic:Del_Shopping_List(i);
			end;
		end;
	end;
end;

--function SellOMatic_List:Apply_iLevel()
--	local list = SellOMatic:Get_Shopping_List();
--	local i;
--	for i=1, #list, 1 do
--		if list[i][3] ~= 0 then
--			if list[i][4] < SellOMatic.db.profile.list_iLevelValue then
--				SellOMatic:Del_Shopping_List(i);
--			end;
--		end;
--	end;
--end;

function SellOMatic_List:Check_Item(list,item)
	local name;
	if list[1] ~= nil then
		for i,v in ipairs(list) do
			if SellOMatic.db.profile.caseSensitiveList then
				name = string.lower(list[i]);
				item = string.lower(item);
			else
				name = list[i];
			end;
			if string.find(item,name) then
				return 1,i;
			end;
		end;
	end;
	return 0;
end;

function SellOMatic:ShowSOMList()
	SellOMatic_List:ShowList(SellOMatic.db.profile.sellList,"sell list");
	SellOMatic_List:ShowList(SellOMatic.db.profile.saveList,"save list");
end;

function SellOMatic_List:ShowList(list,list_type)
	if list[1]~= nil then
		SellOMatic:Print("#### "..list_type.." ####");
		for i,v in ipairs(list) do
			SellOMatic:Print("["..i.."] "..v);
		end;
	else
		SellOMatic:Print(list_type.." empty!");
	end;
end;

function SellOMatic:AddItem(a,item)
	SellOMatic_List:Add_To_List(SellOMatic.db.profile.sellList,"sell list",item);
end;

function SellOMatic_List:Add_To_List(list,list_type,item)
	if item ~= nil and string.len(item) > 0 then
		if SellOMatic_List:Check_Item(list,item) == 0 then
			table.insert(list,item);
			SellOMatic:Print(item.." ".."added to".." "..list_type);
		else
			SellOMatic:Print(item.." ".."already on".." "..list_type);
		end;
	end;
end;

function SellOMatic_List:Del_From_List(list,list_type,item)
	if item ~= nil and string.len(item) > 0 then
		local found,i = SellOMatic_List:Check_Item(list,item);
		local num;
		while found > 0 do
			table.remove(list,i);
			found,i = SellOMatic_List:Check_Item(list,item);
		end;
		if num > 0 then
			SellOMatic:Print(item.." ".."removed from".." "..list_type);
		else
			SellOMatic:Print(item.." ".."not found on".." "..list_type);
		end;
	end;
end;
